// ChatClient.java
//
// Informatics 45 Spring 2010
// Code Example: A Simple Socket Client and Server
//
// This is the program that acts as the client.  It first asks the user to
// specify the address and port of the server that it should connect to.
// It then establishes a connection and begins participating in our simple
// protocol.  Each time it needs to send a message, it asks the user to
// specify the message to be sent.
//
// This program makes no realistic attempt to handle any of the problems
// that might arise while it runs.  It just lets any input/output exception
// be thrown, which will cause the program crash.  This is obviously not
// a realistic way to write a robust program, but it serves our purpose
// here, which is to provide a simple demonstration of using sockets.

package inf45.spring2010.socket1;

import java.io.IOException;
import java.io.PrintWriter;
import java.net.Socket;
import java.util.Scanner;


public class ChatClient
{
	public static void main(String[] args)
	throws IOException
	{
		// We'll need a scanner so that we can read console input from the
		// user.  Notice that we've wrapped this one around System.in,
		// because we want to use it to read console input.
		Scanner consoleIn = new Scanner(System.in);

		// We'll ask the user to specify the address and port that this
		// client should connect to.
		System.out.print("Address: ");
		String address = consoleIn.nextLine();
		
		System.out.print("Port: ");
		int port = Integer.parseInt(consoleIn.nextLine());

		System.out.println("Connecting");

		// When you create a Socket object and specify its address (a string)
		// and port (an integer), it immediately attempts to establish a
		// connection to that location.  Note that this operation, as well as
		// any other that uses the socket, may block indefinitely -- your
		// program has no control over how long it will take to establish
		// the connection, and it will just sit and wait until the connection
		// is made.  (It may eventually give up -- or "time out," as it's more
		// commonly called.  But it may be a while.)  This can be a problem,
		// especially for programs with a graphical user interface that needs
		// to remain responsive.  There are ways to handle this problem, which
		// we'll learn about later.
		Socket socket = new Socket(address, port);

		// Once we've got our Socket, we're connected!  That means we have
		// two streams associated with that socket: the "input stream",
		// where data from the other machine will arrive, and the "output
		// stream", where we'll put data that we want to be sent to the
		// other machine.
		//
		// To make the streams simpler to use, since our protocol is all
		// based on text, we'll wrap them with objects that know how to
		// handle text input and output well.  Note that these are the
		// same objects (Scanners and PrintWriters) that you can use to
		// simplify reading and writing text to and from files.  One of
		// the nice things about Scanners and PrintWriters is that they
		// don't care where they get their input or where they write their
		// output; this makes them flexible enough to use in many scenarios,
		// including this one.
		//
		// One important little detail here: When we create the PrintWriter,
		// we give it two parameters: the stream (which it will "wrap,"
		// by sending any data to it when it's written) and a boolean
		// "true" value.  The second parameter is called "autoFlush."
		// Here's what it means: When data is written to a PrintWriter
		// ordinarily, it is "buffered," which means that it's held in
		// memory until there's a whole bunch of it, then is finally
		// written out to the target (say, a file).  This is wise, because
		// writing output is slow.  Unfortunately, this will play havoc
		// with our protocol, because we want to be absolutely sure that
		// any data we place into the PrintWriter is immediately placed
		// into the socket's output stream, so that it will immediately
		// be sent to the server.  (This is important: remember that the
		// server is sitting and waiting -- indefinitely! -- for our data
		// to show up.)
		Scanner socketIn = new Scanner(socket.getInputStream());
		PrintWriter socketOut = new PrintWriter(socket.getOutputStream(), true);
		
		System.out.println("Connected to port " + port + " at " + address);

		// Each time through the loop, we're reading a message from the console
		// and sending it to the server, then reading the response from the
		// socket and printing it to the console.  The madness ends when one
		// side sends an empty message or response, in which case we send an
		// empty string to the other side (to make sure that it also knows to
		// stop) and then bail out of this loop.
		
		while (true)
		{
			System.out.print("Message: ");

			String message = consoleIn.nextLine();
			
			if (message.length() == 0)
			{
				System.out.println("Ending conversation");
				socketOut.println("");
				break;
			}
			
			socketOut.println(message);

			// At this point, the program will pause indefinitely, waiting for
			// the response to arrive from the server.

			String response = socketIn.nextLine();
			
			if (response.length() == 0)
			{
				System.out.println("Server ended conversation");
				break;
			}
			
			System.out.println("Response: " + response);
		}
		
		System.out.println("Closing connection");

		// It's important that we close the socket when we're done with it.
		// We'll do that here.
		socket.close();
	}
}
